import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { CreateAdCommand } from './create-ad.command';
import { Inject } from '@nestjs/common';
import { AD_REPOSITORY, AD_ROUTE_PROVIDER } from '@modules/ad/ad.di-tokens';
import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import { AdRepositoryPort } from '../../ports/ad.repository.port';
import { AggregateID, ConflictException } from '@mobicoop/ddd-library';
import { AdAlreadyExistsException } from '@modules/ad/core/domain/ad.errors';
import { RouteProviderPort } from '../../ports/route-provider.port';
import { Role } from '@modules/ad/core/domain/ad.types';
import { Route } from '../../types/route.type';

@CommandHandler(CreateAdCommand)
export class CreateAdService implements ICommandHandler {
  constructor(
    @Inject(AD_REPOSITORY)
    private readonly repository: AdRepositoryPort,
    @Inject(AD_ROUTE_PROVIDER)
    private readonly routeProvider: RouteProviderPort,
  ) {}

  async execute(command: CreateAdCommand): Promise<AggregateID> {
    const roles: Role[] = [];
    if (command.driver) roles.push(Role.DRIVER);
    if (command.passenger) roles.push(Role.PASSENGER);
    const route: Route = await this.routeProvider.getBasic(
      roles,
      command.waypoints,
    );
    const ad = AdEntity.create({
      id: command.id,
      driver: command.driver,
      passenger: command.passenger,
      frequency: command.frequency,
      fromDate: command.fromDate,
      toDate: command.toDate,
      schedule: command.schedule,
      seatsProposed: command.seatsProposed,
      seatsRequested: command.seatsRequested,
      strict: command.strict,
      waypoints: command.waypoints,
      points: route.points,
      driverDistance: route.driverDistance,
      driverDuration: route.driverDuration,
      passengerDistance: route.passengerDistance,
      passengerDuration: route.passengerDuration,
      fwdAzimuth: route.fwdAzimuth,
      backAzimuth: route.backAzimuth,
    });

    try {
      await this.repository.insertExtra(ad, 'ad');
      return ad.id;
    } catch (error: any) {
      if (error instanceof ConflictException) {
        throw new AdAlreadyExistsException(error);
      }
      throw error;
    }
  }
}
